//
//  NSBezierPath+MINTLBoxShadow.swift
//  BoxShadowExample
//
//  Created by narita on 2018/11/26.
//  Copyright © 2018 narita. All rights reserved.
//

import Foundation
import Cocoa


// 影を位置の指定に使う。矩形の内側にあるか(くり抜いて見える)、外側にあるか(浮いて見える)

enum MINTLBoxShadowSide : Int
{
    case inside  =  1 // 内側
    case neutral =  0 // 中間
    case outside = -1 // 外側
}

// 矩形の上下左右の辺のMINTLBoxShadowSideの構造体

struct MINTLBoxShadowSides
{
    public init(){}

    public init(   top: MINTLBoxShadowSide,
                bottom: MINTLBoxShadowSide,
                  left: MINTLBoxShadowSide,
                 right: MINTLBoxShadowSide)
    {
        self.top = top
        self.bottom = bottom
        self.left = left
        self.right = right
    }
    
    var top:    MINTLBoxShadowSide = .neutral
    var bottom: MINTLBoxShadowSide = .neutral
    var right:  MINTLBoxShadowSide = .neutral
    var left:   MINTLBoxShadowSide = .neutral
    
    var topRight: MINTLBoxShadowSide
    {
        get { return top == right ? top : .neutral }
    }

    var topLeft: MINTLBoxShadowSide
    {
        get { return top == left ? top : .neutral }
    }

    var bottomRight: MINTLBoxShadowSide
    {
        get { return bottom == right ? bottom : .neutral }
    }
    
    var bottomLeft: MINTLBoxShadowSide
    {
        get { return bottom == left ? bottom : .neutral }
    }
}




extension NSBezierPath
{
    // 外側に影を書くためのパス
    
    class func boxOutsideShadowPath( rect: NSRect,
                                    depth: CGFloat,
                                    sides: MINTLBoxShadowSides) -> NSBezierPath
    {
        let thePoints = NSPointArray.allocate(capacity: 8)

        let theInnerRect  = rect.insetBy(dx:  depth, dy:  depth)
        let theSrcRect    = rect
        
        // 左上の点から時計回りに決定してゆく

        // 上辺
        thePoints[0] = sides.topLeft  == .outside ? theSrcRect.topLeft     : theInnerRect.topLeft
        thePoints[1] = sides.top      == .outside ? theSrcRect.topMiddle   : theInnerRect.topMiddle
        thePoints[2] = sides.topRight == .outside ? theSrcRect.topRight    : theInnerRect.topRight

        // 右辺
        thePoints[3] = sides.right    == .outside ? theSrcRect.middleRight : theInnerRect.middleRight

        // 底辺
        thePoints[4] = sides.bottomRight == .outside ? theSrcRect.bottomRight  : theInnerRect.bottomRight
        thePoints[5] = sides.bottom      == .outside ? theSrcRect.bottomMiddle : theInnerRect.bottomMiddle
        thePoints[6] = sides.bottomLeft  == .outside ? theSrcRect.bottomLeft   : theInnerRect.bottomLeft

        // 左辺
        thePoints[7] = sides.left        == .outside ? theSrcRect.middleLeft   : theInnerRect.middleLeft

        // 決定した点をぐるりと結んで多角形を作る
        let theResult = NSBezierPath()
        
        theResult.appendPoints(thePoints, count: 8)
        theResult.close()
        
        thePoints.deallocate()

        return theResult
    }

    // 内側に影を書くためのパス
    class func boxInsideShadowPath( rect: NSRect,
                                    depth: CGFloat,
                                    sides: MINTLBoxShadowSides) -> NSBezierPath
    {
        let thePoints = NSPointArray.allocate(capacity: 8)
        
        
        let theOutterRect = rect.insetBy(dx: -depth, dy: -depth)
        let theSrcRect = rect
        
        // 左上の点から時計回りに決定してゆく
        
        // 上辺
        thePoints[0] = sides.topLeft  == .inside ? theSrcRect.topLeft      : theOutterRect.topLeft
        thePoints[1] = sides.top      == .inside ? theSrcRect.topMiddle    : theOutterRect.topMiddle
        thePoints[2] = sides.topRight == .inside ? theSrcRect.topRight     : theOutterRect.topRight
        
        // 右辺
        thePoints[3] = sides.right    == .inside ? theSrcRect.middleRight  : theOutterRect.middleRight
        
        // 底辺
        thePoints[4] = sides.bottomRight == .inside ? theSrcRect.bottomRight  : theOutterRect.bottomRight
        thePoints[5] = sides.bottom      == .inside ? theSrcRect.bottomMiddle : theOutterRect.bottomMiddle
        thePoints[6] = sides.bottomLeft  == .inside ? theSrcRect.bottomLeft   : theOutterRect.bottomLeft
        
        // 左辺
        thePoints[7] = sides.left     == .inside ? theSrcRect.middleLeft    : theOutterRect.middleLeft

        // 決定した点をぐるりと結んで多角形を作る
        let theResult = NSBezierPath()
        
        theResult.appendPoints(thePoints, count: 8)
        theResult.close()
        
        thePoints.deallocate()
        
        theResult.appendRect(theOutterRect)
        theResult.windingRule = .evenOdd

        return theResult
    }

    
    
    // 影を書くための、クリッピングパス
    
    class func boxClippingPath( rect: NSRect, depth: CGFloat) -> NSBezierPath
    {
        let theResult = NSBezierPath(rect: rect)
        
        theResult.appendRect(rect.insetBy(dx: depth, dy: depth))
        theResult.windingRule = .evenOdd

        return theResult
    }

    
}
